import torch.nn as nn

class CustomDataParallel(nn.DataParallel):
    def __getattr__(self, name):
        try:
            return super().__getattr__(name)
        except AttributeError:
            return getattr(self.module, name)


import logging
import matplotlib.pyplot as plt
import numpy as np
import os
import pickle
import time
import torch
from collections.abc import Iterable
from sklearn.cluster import KMeans, MiniBatchKMeans, kmeans_plusplus
from sklearn.decomposition import PCA, TruncatedSVD


logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

def pca_plot(X_pca, path):
    plt.figure(figsize=(15,10))
    plt.scatter(X_pca[:,0], X_pca[:,1], s=.9, c='black')
    plt.xticks([])
    plt.yticks([])
    plt.title('Training data projected onto first two components')
    plt.savefig(path)
    log.info('Plot rendered.')
    return

class KMeansInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, k, minibatch, use_logit, batch_size=200, seed=42, path=''):
        if minibatch and use_logit:
            if os.path.exists(path):
                log.info('Loading k-means cluster centroids from precomputed tensor.')
                return torch.load(path)
        log.info('Computing k-Means cluster centroids...')
        if not isinstance(k, Iterable):
            if minibatch:
                k_means_obj = MiniBatchKMeans(n_clusters=k, batch_size=batch_size, random_state=seed, n_init=3, max_iter=3).fit(X)
            else:
                k_means_obj = KMeans(n_clusters=k, random_state=42, n_init=3, max_iter=3).fit(X)
            if not use_logit:
                return torch.tensor(k_means_obj.cluster_centers_).float()
            P_init = torch.clamp(torch.tensor(k_means_obj.cluster_centers_).float(), min=1e-4, max=1-1e-4) 
            del k_means_obj
            P_init = torch.logit(P_init, eps=1e-4)
            if sum(torch.isnan(P_init.flatten())).item() > 0:
                log.error('Initialization weights contain NaN values.')
                raise Exception
            return P_init
        else:
            log.info(f'Running {len(k)} algorithms (multihead)...')
            k_means_objs = [None]*len(k)
            if minibatch:
                k_means_objs = [MiniBatchKMeans(n_clusters=i, batch_size=batch_size, random_state=seed, n_init=3, max_iter=3, compute_labels=False).fit(X)
                                for i in k]
            else:
                k_means_objs = [KMeans(n_clusters=i, random_state=42, n_init=3, max_iter=3).fit(X) for i in k]
            if not use_logit:
                return torch.cat([torch.tensor(obj.cluster_centers_) for obj in k_means_objs], axis=0).float()
            P_init = torch.clamp(torch.cat([torch.tensor(obj.cluster_centers_) for obj in k_means_objs], axis=0).float(),
                                 min=1e-4, max=1-1e-4)
            del k_means_objs
            P_init = torch.logit(P_init, eps=1e-4)
            if sum(torch.isnan(P_init.flatten())).item() > 0:
                log.error('Initialization weights contain NaN values.')
                raise Exception
            torch.save(P_init, path)
            log.info('All k-means run successfully. Initialization was saved.')
            return P_init


class KMeansPlusPlusInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, k, seed=42, path=''):
        if os.path.exists(path):
            log.info('Loading k-means++ centroids from precomputed tensor.')
            return torch.load(path)
        log.info('Computing k-Means++ centroids...')
        X_np = np.array(X[:])
        if not isinstance(k, Iterable):
            centers, _ = kmeans_plusplus(X_np, n_clusters=k, random_state=seed)
            P_init = torch.clamp(torch.tensor(centers).float(), min=1e-4, max=1-1e-4) 
            del centers, X_np
            if sum(torch.isnan(P_init.flatten())).item() > 0:
                log.error('Initialization weights contain NaN values.')
                raise Exception
            return P_init
        else:
            log.info(f'Running {len(k)} algorithms (multihead)...')
            centers = [kmeans_plusplus(X_np, n_clusters=i, random_state=seed)[0]
                       for i in k]
            P_init = torch.clamp(torch.cat([torch.tensor(obj) for obj in centers], axis=0).float(),
                                 min=1e-4, max=1-1e-4)
            del centers, X_np
            if sum(torch.isnan(P_init.flatten())).item() > 0:
                log.error('Initialization weights contain NaN values.')
                raise Exception
            torch.save(P_init, path)
            log.info('All k-means++ run successfully. Initialization was saved.')
            return P_init

class RandomMeanInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K):
        X_mean = torch.tensor(np.mean(X, axis=0)).unsqueeze(1)
        P_init = (torch.bernoulli(X_mean.repeat(1, K))-0.5).T.float()
        return P_init

class SNPsMeanInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K):
        X_mean = torch.tensor(np.mean(X, axis=0), dtype=torch.float)
        return X_mean.repeat(K, 1)

class BinomialInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K, path=''):
        if os.path.exists(path):
            log.info('Loading binomial initialization from precomputed tensor.')
            return torch.load(path)
        log.info(f'Running binomial initialization...')
        np.random.seed(42)
        P_inits = [torch.tensor(np.random.binomial(n=1, p=0.1, size=X.shape[1]), dtype=torch.float) for _ in range(sum(K))]
        P_init = torch.logit(torch.vstack(P_inits), eps=1e-4)
        del P_inits
        torch.save(P_init, path)
        return P_init

class PCAInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K, path=''):
        if len(K) > 1:
            raise NotImplementedError
        k = K[0]
        if k <= 4 or k > 9:
            raise NotImplementedError
        log.info('Running PCA initialization...')
        try:
            with open(path, 'rb') as fb:
                pca_obj = pickle.load(fb)
            log.info('PCA loaded.') 
        except FileNotFoundError as fnfe:
            log.info('PCA object not found. Performing PCA...')
            pca_obj = PCA(n_components=2, random_state=42)
            pca_obj.fit(X)
            with open(path, 'wb') as fb:
                pickle.dump(pca_obj, fb)
        except Exception as e:
            raise e
        X_pca = pca_obj.transform(X)
        svs = np.sqrt(pca_obj.singular_values_)
        part1 = np.array([-svs[0]/2,0,svs[0]/2])
        part2 = np.array([-svs[1]/2,0,svs[1]/2])
        xv, yv = np.meshgrid(part1, part2)
        concat = np.concatenate((xv.reshape(-1,1), yv.reshape(-1,1)), axis=1)
        init_points = [None]*len(concat)
        for i, p in enumerate(concat):
            rep = np.vstack([p]*X.shape[0])
            init_points[i] = X_pca[np.argmin(np.linalg.norm(rep-X_pca, axis=1))]
        init_points = np.vstack(init_points)
        np.random.seed(42)
        inv_tr = pca_obj.inverse_transform(init_points)
        idxs = np.random.choice(inv_tr.shape[0], k, replace=False)
        P_init = torch.tensor(inv_tr[idxs,:], dtype=torch.float)
        log.info('Weights initialized.')
        return P_init

class AdmixtureInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K, path):
        log.info('Fetching ADMIXTURE weights...')
        if len(K) > 1:
            raise NotImplementedError
        # Loads standard ADMIXTURE output format
        P_init = torch.tensor(1-np.genfromtxt(path, delimiter=' ').T, dtype=torch.float32)
        assert P_init.size()[0] == K[0], 'Input P is not coherent with the value of K'
        log.info('Weights fetched.')
        return P_init

class PCKMeansInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K, path, run_name, n_components):
        log.info('Running PC-KMeans initialization...')
        t0 = time.time()
        try:
            with open(path, 'rb') as fb:
                pca_obj = pickle.load(fb)
            log.info('PCA loaded.')
            if pca_obj.n_components_ != n_components:
                raise FileNotFoundError
        except FileNotFoundError as fnfe:
            log.info(f'{n_components}D PCA object not found. Performing PCA...')
            pca_obj = PCA(n_components=n_components, random_state=42)
            pca_obj.fit(X)
            with open(path, 'wb') as fb:
                pickle.dump(pca_obj, fb)
        except Exception as e:
            raise e
        assert pca_obj.n_features_ == X.shape[1], 'Computed PCA and training data do not have same number of SNPs' 
        log.info('Projecting data...')
        X_pca = pca_obj.transform(X)
        log.info('Running KMeans on projected data...')
        if isinstance(K, Iterable):
            k_means_objs = [KMeans(n_clusters=i, random_state=42, n_init=10, max_iter=10).fit(X_pca) for i in K]
            centers = np.concatenate([obj.cluster_centers_ for obj in k_means_objs])
            P_init = torch.tensor(pca_obj.inverse_transform(centers), dtype=torch.float32).view(sum(K), -1)
        else:
            k_means_obj = KMeans(n_clusters=K, random_state=42, n_init=10, max_iter=10).fit(X_tsvd)
            P_init = torch.tensor(pca_obj.inverse_transform(k_means_obj.cluster_centers_), dtype=torch.float32).view(K, -1)
        te = time.time()
        log.info('Weights initialized in {} seconds.'.format(te-t0))
        log.info('Rendering PCA plot...')
        try:
            save_root = '/'.join(path.split('/')[:-1]) if len(path.split('/')) > 1 else '.'
            plot_save_path = f'{save_root}/{run_name}_training_pca.png'
            pca_plot(X_pca, plot_save_path)
        except Exception as e:
            log.warn(f'Could not render PCA plot: {e}')
            log.info('Resuming...')
        return P_init


class TSVDKMeansInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, K, path, run_name, n_components):
        log.info('Running TSVD-KMeans initialization...')
        t0 = time.time()
        try:
            with open(path, 'rb') as fb:
                tsvd_obj = pickle.load(fb)
            log.info('TSVD loaded.')
            if tsvd_obj.components_.shape[0] != n_components:
                raise FileNotFoundError
        except FileNotFoundError as fnfe:
            log.info(f'{n_components}D TSVD object not found. Performing TSVD...')
            tsvd_obj = TruncatedSVD(n_components=n_components, random_state=42)
            tsvd_obj.fit(X)
            with open(path, 'wb') as fb:
                pickle.dump(tsvd_obj, fb)
        except Exception as e:
            raise e
        assert tsvd_obj.components_.shape[1] == X.shape[1], 'Computed TSVD and training data do not have same number of SNPs' 
        log.info('Projecting data...')
        X_tsvd = tsvd_obj.transform(X)
        log.info('Running KMeans on projected data...')
        if isinstance(K, Iterable):
            k_means_objs = [KMeans(n_clusters=i, random_state=42, n_init=10, max_iter=10).fit(X_tsvd) for i in K]
            centers = np.concatenate([obj.cluster_centers_ for obj in k_means_objs])
            P_init = torch.tensor(tsvd_obj.inverse_transform(centers), dtype=torch.float32).view(sum(K), -1)
        else:
            k_means_obj = KMeans(n_clusters=K, random_state=42, n_init=10, max_iter=10).fit(X_tsvd)
            P_init = torch.tensor(tsvd_obj.inverse_transform(k_means_obj.cluster_centers_), dtype=torch.float32).view(K, -1)
        te = time.time()
        log.info('Weights initialized in {} seconds.'.format(te-t0))
        log.info('Rendering TSVD plot...')
        try:
            save_root = '/'.join(path.split('/')[:-1]) if len(path.split('/')) > 1 else '.'
            plot_save_path = f'{save_root}/{run_name}_training_tsvd.png'
            pca_plot(X_tsvd, plot_save_path)
        except Exception as e:
            log.warn(f'Could not render TSVD plot: {e}')
            log.info('Resuming...')
        return P_init
        
class SupervisedInitialization(object):
    @classmethod
    def get_decoder_init(cls, X, y, K):
        log.info('Running supervised initialization...')
        assert y is not None, 'Ground truth ancestries needed for supervised mode'
        if len(K) > 1:
            raise NotImplementedError
        t0 = time.time()
        k = K[0]
        ancestry_dict = {anc: idx for idx, anc in enumerate(sorted(np.unique(y)))}
        assert len(ancestry_dict) == k, 'Number of ancestries in training ground truth is not equal to the value of k'
        to_idx_mapper = np.vectorize(lambda x: ancestry_dict[x])
        y_num = to_idx_mapper(y[:])
        X_mem = X[:,:]
        P_init = torch.tensor(np.vstack([X_mem[y_num==idx,:].mean(axis=0) for idx in range(k)]), dtype=torch.float32)
        te = time.time()
        log.info('Weights initialized in {} seconds.'.format(te-t0))
        return P_init


import sys
sys.path.append('..')
import model.initializations as init
import torch.nn as nn
import torch.optim as optim

class Switchers(object):
    _activations = {
        'relu': lambda x: nn.ReLU(),
        'tanh': lambda x: nn.Tanh()
    }


    _data = {
        'CHM-1': lambda path: (f'{path}/CHM-1/train.h5', f'{path}/CHM-1/validation.h5'),
        'CHM-22': lambda path: (f'{path}/CHM-22/train.h5', f'{path}/CHM-22/validation.h5'),
        'CHM-22-SIM': lambda path: (f'{path}/CHM-22-SIM/train.h5', f'{path}/CHM-22-SIM/validation.h5')
    }

    _initializations = {
        'random': lambda X, y, k, batch_size, seed, path, run_name, n_comp: None,
        'mean_SNPs': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.SNPsMeanInitialization.get_decoder_init(X, k),
        'mean_random': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.RandomMeanInitialization.get_decoder_init(X, k),
        'kmeans': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.KMeansInitialization.get_decoder_init(X, k, False, False, batch_size, seed),
        'minibatch_kmeans': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.KMeansInitialization.get_decoder_init(X, k, True, False, batch_size, seed),
        'kmeans++': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.KMeansPlusPlusInitialization.get_decoder_init(X, k, seed),
        'binomial': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.BinomialInitialization.get_decoder_init(X, k),
        'pca': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.PCAInitialization.get_decoder_init(X, k),
        'admixture': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.AdmixtureInitialization.get_decoder_init(X, k, path),
        'pckmeans': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.PCKMeansInitialization.get_decoder_init(X, k, path, run_name, n_comp),
        'supervised': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.SupervisedInitialization.get_decoder_init(X, y, k),
        'tsvdkmeans': lambda X, y, k, batch_size, seed, path, run_name, n_comp: init.TSVDKMeansInitialization.get_decoder_init(X, k, path, run_name, n_comp)
    }

    _optimizers = {
        'adam': lambda params, lr: optim.Adam(params, lr),
        'sgd': lambda params, lr: optim.SGD(params, lr)
    }

    @classmethod
    def get_switchers(cls):
        return {
            'activations': cls._activations,
            'data': cls._data,
            'initializations': cls._initializations,
            'optimizers': cls._optimizers
        }



import argparse
import gc
import logging
import os
import subprocess
import sys
from pathlib import Path

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--data_path', required=False, default='../data', type=str, help='Path where data is stored')
    return parser.parse_args()

def main():
    args = parse_args()
    data_path = args.data_path
    log.info('Training...')
    sps = os.system(f'python3 train.py --display_logs 1 --epochs 10 --decoder_init pckmeans --name CHM-22 --save_dir ../outputs/weights --data_path {data_path} --dataset CHM-22 --linear 1 --init_path ../outputs/initializations/CHM-22_PCA.pkl')
    if sps != 0:
        return sps
    log.info('1/3 models trained.')
    del sps
    gc.collect()
    sps = os.system(f'python3 train.py --display_logs 1 --epochs 10 --decoder_init admixture --name CHM-22-PRETRAINED --save_dir ../outputs/weights --data_path {data_path} --dataset CHM-22 --linear 1 --init_path {data_path}/CHM-22/CHM-22_classic_train.P --freeze_decoder')
    if sps != 0:
        return sps
    log.info('2/3 models trained.')
    del sps
    gc.collect()
    sps = os.system(f'python3 train.py --display_logs 1 --epochs 6 --decoder_init supervised --name CHM-22-SUPERVISED --save_dir ../outputs/weights --data_path {data_path} --dataset CHM-22 --linear 1 --freeze_decoder --supervised')
    if sps != 0:
        return sps
    log.info('3/3 models trained.')
    del sps
    return 0

if __name__ == '__main__':
    Path("../outputs/weights").mkdir(parents=True, exist_ok=True)
    Path("../outputs/initializations").mkdir(parents=True, exist_ok=True)
    Path("../outputs/figures").mkdir(parents=True, exist_ok=True)
    sys.exit(main())


import allel
import argparse
import h5py
import logging
import numpy as np
import sys

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--vcf', required=True, type=str, help='Path of input VCF file.')
    parser.add_argument('--out', required=False, type=str, default='', help='Desired output path for HDF5 file.')
    return parser.parse_args()

def convert_to_hdf5(args):
    vcf_path = args.vcf
    out_path = args.out if args.out != '' else vcf_path.split('.vcf')[0]
    log.info('Reading VCF file...')
    vcf_f = allel.read_vcf(vcf_path)
    log.info('Processing...')
    np_ar = np.sum(vcf_f['calldata/GT'], axis=2).T/2
    log.info('Storing HDF5 file...')
    h5f = h5py.File(out_path, 'w')
    h5f.create_dataset('snps', data=np_ar)
    h5f.close()
    log.info('HDF5 file created.')
    return np_ar
    
def main(args):
    return convert_to_hdf5(args)

if __name__ == '__main__':
    args = parse_args()
    sys.exit(main(args))



from setuptools import find_packages, setup


setup(
    name='neural-admixture',
    version='0.5.0',
    description='Population clustering with autoencoders',
    url='https://github.com/AI-sandbox/neural-admixture',
    author='Albert Dominguez Mantes',
    author_email='adomi@stanford.edu',
    scripts=['./neural-admixture/neural-admixture'],
    license='CC BY-NC 4.0',
    packages=find_packages('neural-admixture/')+['.'],
    package_dir={"": "neural-admixture"},
    include_package_data=True,
    install_requires=['codetiming==1.3.0',
                      'h5py==3.1.0',
                      'matplotlib==3.3.4',
                      'pandas==1.2.4',
                      'pandas_plink==2.2.9',
                      'scikit-allel==1.3.5',
                      'scikit-learn==0.24.1',
                      'setuptools==50.3.1',
                      'torch==1.7.1',
                      'wandb==0.10.21'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



import argparse
import logging
import h5py
import numpy as np
import pandas as pd
import sys
import torch
from sklearn.preprocessing import OneHotEncoder
from sklearn.metrics import adjusted_mutual_info_score
from switchers import Switchers
sys.path.append('..')
from model.neural_admixture import NeuralAdmixture

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--data_path', required=False, default='../data', type=str, help='Path to data')
    parser.add_argument('--weights_path', required=False, default='../outputs/weights', type=str, help='Path to trained weights')
    return parser.parse_args()

def get_model_preds(model, data, device):
    model.to(device)
    outs = torch.tensor([])
    model.eval()
    with torch.no_grad():
        for i, (X, _) in enumerate(model._batch_generator(data, 200, shuffle=False)):
            X = X.to(device)
            out = model(X, True)
            outs = torch.cat((outs, out[0].detach().cpu()), axis=0)
    return outs.cpu().numpy()

def print_losses(model, data, Qs_adm, Ps_adm, device, loss_f=torch.nn.BCELoss(reduction='sum')):
    preds_adm = Qs_adm@Ps_adm
    loss_class = 0
    loss_neural = 0 
    model.eval()
    with torch.no_grad():
        for i, (X, _) in enumerate(model._batch_generator(data, 200, shuffle=False)):
            X = X.to(device)
            recs, _ = model(X)
            loss_neural += loss_f(X, recs[0]).item()
            loss_class += loss_f(X, torch.tensor(preds_adm[200*i:200*i+200], dtype=torch.float, device=device)).item()
            del recs
    log.info('Classical loss: {:e}'.format(loss_class))
    log.info('Neural ADMIXTURE loss: {:e}'.format(loss_neural))
    log.info('------------------------------------------------------')

def output_metrics(gt, preds_class, preds_neur):
    assert len(gt) == len(preds_class) == len(preds_neur), 'GT and predictions do not have same number of samples'
    class_assign = np.argmax(preds_class, axis=1)
    neur_assign = np.argmax(preds_neur, axis=1)
    gt_oh = OneHotEncoder().fit_transform(gt.reshape(-1,1))
    log.info('Classical Mean Squared Second Order Difference (Delta): {}'.format(((preds_class@preds_class.T-gt_oh@gt_oh.T)**2).mean()))
    log.info('Neural Mean Squared Second Order Difference (Delta): {}'.format(((preds_neur@preds_neur.T-gt_oh@gt_oh.T)**2).mean()))
    log.info('------------------------------------------------------')
    log.info('Classical Adjusted Mutual Information score (AMI): {}'.format(adjusted_mutual_info_score(gt, class_assign)))
    log.info('Neural Adjusted Mutual Information score(AMI): {}'.format(adjusted_mutual_info_score(gt, neur_assign)))
    log.info('------------------------------------------------------')

def main():
    args = parse_args()
    data_path = args.data_path
    dataset = 'CHM-22'
    switchers = Switchers.get_switchers()
    tr_file, val_file = switchers['data']['CHM-22'](data_path)
    f_tr = h5py.File(tr_file, 'r')
    f_val = h5py.File(val_file, 'r')
    trX, trY, valX, valY = f_tr['snps'], f_tr['populations'], f_val['snps'], f_val['populations']
    device = torch.device('cuda:0') if torch.cuda.is_available() else torch.device('cpu')
    name = 'CHM-22'
    log.info(f'---------------------- {name} ----------------------')
    model_path = f'{args.weights_path}/{name}.pt'
    model = NeuralAdmixture([7], num_features=trX.shape[1])
    model.load_state_dict(torch.load(model_path, map_location=torch.device('cpu')), strict=True)
    model.to(device)
    log.info('Computing training metrics...')
    Qs_adm_tr = pd.read_csv(f'{data_path}/{dataset}/{name}_classic_train.Q', sep=' ', names=np.array(range(7))).to_numpy()
    Ps_adm_tr = 1-pd.read_csv(f'{data_path}/{dataset}/{name}_classic_train.P', sep=' ', names=np.array(range(7))).to_numpy().T
    tr_outputs = get_model_preds(model, trX, device)
    output_metrics(trY[:], Qs_adm_tr, tr_outputs)
    log.info('Computing validation metrics...')
    Qs_adm_val = pd.read_csv(f'{data_path}/{dataset}/{name}_classic_valid.Q', sep=' ', names=np.array(range(7))).to_numpy()
    Ps_adm_val = 1-pd.read_csv(f'{data_path}/{dataset}/{name}_classic_valid.P', sep=' ', names=np.array(range(7))).to_numpy().T
    val_outputs = get_model_preds(model, valX, device)
    print_losses(model, valX, Qs_adm_val, Ps_adm_val, device)
    output_metrics(valY[:], Qs_adm_val, val_outputs)
    del model
    if torch.cuda.is_available():
        torch.cuda.empty_cache()
        torch.cuda.synchronize()
    name = 'CHM-22-PRETRAINED'
    log.info(f'---------------------- {name} ----------------------')
    model_path = f'{args.weights_path}/{name}.pt'
    model = NeuralAdmixture([7], num_features=trX.shape[1])
    model.load_state_dict(torch.load(model_path, map_location=torch.device('cpu')), strict=True)
    model.to(device)
    log.info('Computing training metrics...')
    Qs_adm_tr = pd.read_csv(f'{data_path}/{dataset}/CHM-22_classic_train.Q', sep=' ', names=np.array(range(7))).to_numpy()
    Ps_adm_tr = 1-pd.read_csv(f'{data_path}/{dataset}/CHM-22_classic_train.P', sep=' ', names=np.array(range(7))).to_numpy().T
    tr_outputs = get_model_preds(model, trX, device)
    output_metrics(trY[:], Qs_adm_tr, tr_outputs)
    log.info('Computing validation metrics...')
    Qs_adm_val = pd.read_csv(f'{data_path}/{dataset}/CHM-22_classic_valid.Q', sep=' ', names=np.array(range(7))).to_numpy()
    Ps_adm_val = 1-pd.read_csv(f'{data_path}/{dataset}/CHM-22_classic_valid.P', sep=' ', names=np.array(range(7))).to_numpy().T
    print_losses(model, valX, Qs_adm_val, Ps_adm_val, device)
    val_outputs = get_model_preds(model, valX, device)
    output_metrics(valY[:], Qs_adm_val, val_outputs)
    if torch.cuda.is_available():
        torch.cuda.empty_cache()
        torch.cuda.synchronize()
    name = 'CHM-22-SUPERVISED'
    log.info(f'---------------------- {name} ----------------------')
    model_path = f'{args.weights_path}/{name}.pt'
    model = NeuralAdmixture([7], num_features=trX.shape[1])
    model.load_state_dict(torch.load(model_path, map_location=torch.device('cpu')), strict=True)
    model.to(device)
    log.info('Computing training metrics...')
    Qs_adm_tr = pd.read_csv(f'{data_path}/{dataset}/{name}_classic_train.Q', sep=' ', names=np.array(range(7))).to_numpy()
    Ps_adm_tr = 1-pd.read_csv(f'{data_path}/{dataset}/{name}_classic_train.P', sep=' ', names=np.array(range(7))).to_numpy().T
    tr_outputs = get_model_preds(model, trX, device)
    output_metrics(trY[:], Qs_adm_tr, tr_outputs)
    log.info('Computing validation metrics...')
    Qs_adm_val = pd.read_csv(f'{data_path}/{dataset}/{name}_classic_valid.Q', sep=' ', names=np.array(range(7))).to_numpy()
    Ps_adm_val = 1-pd.read_csv(f'{data_path}/{dataset}/{name}_classic_valid.P', sep=' ', names=np.array(range(7))).to_numpy().T
    print_losses(model, valX, Qs_adm_val, Ps_adm_val, device)
    val_outputs = get_model_preds(model, valX, device)
    output_metrics(valY[:], Qs_adm_val, val_outputs)
    return 0

if __name__ == '__main__':
    sys.exit(main())